/**
 * 
 */
package edu.westga.model.eventparser;

import twitter4j.Tweet;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.*;

import javax.swing.ImageIcon;

/**
 * @author Michael Obermeyer
 * 		   <Feel free to add your name here if you make changes>
 * 
 * This class contains data abstraction to accept a Tweet object and convert it
 * into an object that can be used to represent an Event.
 *
 */
public class Event {

	private Date eventDate;
	private Date postDate;
	private String description;
	private String fromUser;
	private String toUser;
	private String postLocation;
	private ImageIcon profileImageIcon;
	private Tweet tweet;
	
	/**
	 * Creates an instance of an Event object with the given Tweet.
	 * A Tweet, currently, is considered an event if and only if there
	 * is a Date discovered within its text. This is different from having
	 * a PostDate. If the content of the message has a date, then it is
	 * assumed to refer to an event on that date. If no event is found,
	 * null is returned.
	 * 
	 * Ideally, this class will contain a way to retrieve the total number
	 * of retweets for a given event in the future.
	 * 
	 * 
	 * @param theTweet: The Tweet to convert into an event.
	 */
	public static Event create(Tweet theTweet){
			
		Date aPostDate = theTweet.getCreatedAt();
		Date event = DateParser.parseDate(theTweet.getText(), aPostDate);
		if(event == null){
			return null;
		}
				
		Event theEvent = new Event();
		theEvent.tweet = theTweet; 
		theEvent.eventDate = event;
		theEvent.postDate = aPostDate;
		theEvent.description = theTweet.getText();
		theEvent.fromUser = theTweet.getFromUser();
		theEvent.toUser = theTweet.getToUser();
		theEvent.postLocation = theTweet.getLocation();		
		
		try {
			URL imageUrl = new URL(theTweet.getProfileImageUrl());
			theEvent.profileImageIcon = new ImageIcon(imageUrl);
		} catch (MalformedURLException e) {
			theEvent.profileImageIcon = null;
		}
		
		return theEvent;
	}
	
	/**
	 * Returns the date on which the event is to occur.
	 * 
	 * @return Date: date of the event's occurrence
	 */
	public Date getEventDate(){
		return this.eventDate;
	}
	
	/**
	 * Returns the date on which the tweet was posted.
	 * 
	 * @return Date: date on which the tweet was posted.
	 */
	public Date getPostDate(){
		return this.postDate;
	}
	
	/**
	 * Returns the description of the event. The description is
	 * intended to be the text of the original Tweet.
	 * 
	 * @return String: description of the event.
	 */
	public String getDescription(){
		return this.description;
	}
	
	/**
	 * Retrieves the twitter user that posted the event.
	 * 
	 * @return String: twitter user that posted the event
	 */
	public String getFromUser(){
		return this.fromUser;
	}
	
	/**
	 * Retrieves the twitter user to which the Tweet was sent.
	 * 
	 * @return String: twitter user to which the Tweet was sent.
	 */
	public String getToUser(){
		return this.toUser;
	}
	
	/**
	 * Gets the location of the original tweet.
	 * 
	 * @return String: string representation of the post location.
	 */
	public String getPostLocation(){
		return this.postLocation;
	}
	
	/**
	 * Returns the ImageIcon associated with the original poster.
	 * 
	 * @return ImageIcon: image of the original poster
	 */
	public ImageIcon getProfileImageIcon(){
		return this.profileImageIcon;
	}
	
	/**
	 * Retrieves the Tweet associated with this event.
	 * 
	 * @return Tweet: the Tweet associated with this event.
	 */
	public Tweet getTweet(){
		return this.tweet;
	}
	
	
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~PRIVATE HELPER METHODS~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	
	
	
	//This is private simply to prevent the default constructor from being called.
	//It is used to create an instance of an event and set its values manually.
	private Event(){}
}
